#include "/settings.glsl"

#define UPSCALE_TERRAIN 1 //[0 1]
#define UPSCALE_ENTITIES 1 //[0 1]
#define UPSCALE_HAND_HELD 0 //[0 1]
#define UPSCALE_PARTICLES 0 //[0 1]

uniform sampler2D lightmap;

	#define COLORED_SHADOWS 1 //[0 1 2] //0: Stained glass will cast ordinary shadows. 1: Stained glass will cast colored shadows. 2: Stained glass will not cast any shadows. 
	#define SHADOW_BRIGHTNESS 0.75 //Light levels are multiplied by this number when the surface is in shadows [0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00]

	uniform sampler2D shadowcolor0;
	uniform sampler2D shadowtex0;
	uniform sampler2D shadowtex1;
	varying vec4 shadowPos;
#if SHADOWS == 1
#endif
uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D specular;

uniform float far;
uniform float near;
uniform int isEyeInWater;
uniform float fogStart;
uniform float fogEnd;
uniform vec3 fogColor;
uniform float rainStrength;
uniform float sunAngle;


varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
#if SHOW_MOB_DAMAGE == 1 && IS_AN_ENTITY == 1
	varying vec4 entityColor4;
#endif

varying vec4 viewPos;


//fix artifacts when colored shadows are enabled
const bool shadowcolor0Nearest = true;
const bool shadowtex0Nearest = true;
const bool shadowtex1Nearest = true;

//only using this include for shadowMapResolution,
//since that has to be declared in the fragment stage in order to do anything.
#include "/distort.glsl"




    vec2 dfdx = dFdx(texcoord.st);
	vec2 dfdy = dFdy(texcoord.st);
	varying  vec4 vlocal_uv_components;//CTMPOMFIX
	varying  vec4 vlocal_uv;//CTMPOMFIX
	#include "/timetravelbeards_Better_3D_Textures.glsl"

#if PBR > 0
	varying vec4 tangent;
	varying vec3 normals_face;
	uniform vec3 shadowLightPosition;
	uniform vec3 upPosition;
	#if PBR >=2
		varying float ipbr_id;
	#endif
#endif
#if PBR >=2 || SKY_COLOR_ALTERNATE == 1
		uniform vec3 skyColor;
#endif



void main() {
	
	
	float dist = distance(vec3(0.),viewPos.xyz);
	#if TEXTURE_FILTERING_CPF > 0 && FILTER_HERE == 1
		vec4 color= atlas_uv_to_bilinear_data( texcoord,dist )*glcolor;
		#if PBR > 0
			vec4 normals_pixel = atlas_uv_to_bilinear_data_normal( texcoord,dist );
		#endif
		#if PBR >= 2
			vec4 specular_pixel = atlas_uv_to_bilinear_data_specular( texcoord,dist );
		#endif
	#else
		vec4 color = texture2D(texture, texcoord) * glcolor;
		#if PBR > 0
			vec4 normals_pixel = texture2D(normals, texcoord);
		#endif	
		#if PBR >= 2
			vec4 specular_pixel = texture2D(specular, texcoord);
		#endif
	#endif
	
	
	
	
	float main_lighting = sunAngle <.5 ? SUN_BRIGHTNESS : MOON_BRIGHTNESS ;
	
	
	
	vec2 lm = lmcoord;
	
			//Lighting Primary
						lm.x=pow(lm.x,TORCH_FALLOFF)*TORCH_BRIGHTNESS;
						lm.y=pow(lm.y,SKY_LIGHT_FALLOFF)*SKY_LIGHT_BRIGHTNESS;
						
						
						vec3 sky_color = 
						#if SKY_COLOR_ALTERNATE == 1
							#if EXTRA_DARK_NIGHT > 0
								mix(
								skyColor,
								vec3(0.),
								clamp((1.-abs(sunAngle-.75)*4.)*10.,0.,1.)*float(EXTRA_DARK_NIGHT)*.1
								)
							#else
						skyColor
						#endif
						#else
							#if EXTRA_DARK_NIGHT > 0
								mix(
								texture2D(lightmap, vec2(1./32.,lm.y)).rgb,
								vec3(0.),
								clamp((1.-abs(sunAngle-.75)*4.)*10.,0.,1.)*float(EXTRA_DARK_NIGHT)*.1
								)
							#else
								texture2D(lightmap, vec2(1./32.,lm.y)).rgb
							#endif	
						#endif
						
						
						;
						
						
					#if SUNSET == 0
						vec3 sun_color =
						
						#if CUSTOM_SUN_COLOR == 0
							sky_color
						#endif
						#if CUSTOM_SUN_COLOR == 1
							(sunAngle <.5 ?
							vec3(SUN_COLOR_R,SUN_COLOR_G,SUN_COLOR_B)
							:vec3(MOON_COLOR_R,MOON_COLOR_G,MOON_COLOR_B)
							)
							
						#endif
						;
					#endif
					
					#if SUNSET == 2
						vec3 sun_color =
						clamp(
						
						#if CUSTOM_SUN_COLOR == 0
							sky_color
						#endif
						#if CUSTOM_SUN_COLOR == 1
							(sunAngle <.5 ?
							 vec3(SUN_COLOR_R,SUN_COLOR_G,SUN_COLOR_B)
							 :vec3(MOON_COLOR_R,MOON_COLOR_G,MOON_COLOR_B)
							 )
						#endif

						  -
						  vec3(SUNSET_FADE_R,SUNSET_FADE_G,SUNSET_FADE_B )
						  *(1.-clamp((1.-abs(sunAngle-
						  (sunAngle<.5? .25 : .75)
						  
						  )*4.)*10.,0.,1.))
						  ,0.,1.)
						  ;
					#endif
						 
						 
					#if SUNSET == 1
							#if CUSTOM_SUN_COLOR == 0
								vec3 sun_color = sky_color
							#endif
							#if CUSTOM_SUN_COLOR == 1
								vec3 sun_color =
								sunAngle <.5 ?
								 vec3(SUN_COLOR_R,SUN_COLOR_G,SUN_COLOR_B)
								 :vec3(MOON_COLOR_R,MOON_COLOR_G,MOON_COLOR_B)
							 #endif
						*clamp((1.-abs(sunAngle-
						  (sunAngle<.5? .25 : .75)
						  
						  )*4.)*10.,0.,1.);
				
	 
					#endif
				
					
	
						
						#if CUSTOM_TORCH_COLOR == 0
							vec3 torch_color = 
								vec3(1.,0.9,0.8);
								//texture2D(lightmap, vec2(lm.x,0.1)).rgb;//
						#endif
						#if CUSTOM_TORCH_COLOR == 1
							vec3 torch_color = 
								vec3(TORCH_HI_R,TORCH_HI_G,TORCH_HI_B);
								
						#endif
						#if CUSTOM_TORCH_COLOR == 2
							vec3 torch_color = 
								mix(vec3(TORCH_LOW_R,TORCH_LOW_G,TORCH_LOW_B),
								vec3(TORCH_HI_R,TORCH_HI_G,TORCH_HI_B),
								lm.x)
								;
								
						#endif
						
						
							
								
						
						
						
						vec4 shadowLightColor=vec4(1.0);
						
						
	#if PBR > 0
		vec3 tangent2 = normalize(cross(tangent.rgb,normals_face.xyz)*tangent.w);
		mat3 tbn_matrix = mat3(tangent.xyz, tangent2.xyz, normals_face.xyz);

		
		float ao_allowed_light = mix(1.,normals_pixel.z,AMBIENT_OCCLUSSION_TEXTURES);
		normals_pixel.xy=normals_pixel.xy*2.-1.;
		normals_pixel.z = sqrt(1.0-dot(normals_pixel.xy, normals_pixel.xy)); //Reconstruct Z
		
		normals_pixel.xyz =normalize(normals_pixel.xyz );//fsster no norm?
		normals_pixel.xyz = normalize(tbn_matrix * normals_pixel.xyz); //Rotate by TBN matrix //faster no norm?
		float sun_lighting = clamp (dot(normalize(shadowLightPosition), normals_pixel.xyz) ,0.,1.) ;
		vec3 n_sky_dir = normalize(upPosition);
		float sky_lighting = clamp((dot(n_sky_dir,normals_pixel.xyz)+1.)*.5, NON_DIRECTIONAL_AMBIENT_SKY_LIGHT ,1.);//fix?
		
	#endif
						
						
						
	#if SHADOWS == 1
		
		if (shadowPos.w > 0.0) {
			//surface is facing towards shadowLightPosition
			#if COLORED_SHADOWS == 0
				//for normal shadows, only consider the closest thing to the sun,
				//regardless of whether or not it's opaque.
				if (texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
			#else
				//for invisible and colored shadows, first check the closest OPAQUE thing to the sun.
				if (texture2D(shadowtex1, shadowPos.xy).r < shadowPos.z) {
			#endif
				//surface is in shadows. reduce light level.
				main_lighting = 0.0;
			}
			else {
				//surface is in direct sunlight
				#if COLORED_SHADOWS == 1
					//when colored shadows are enabled and there's nothing OPAQUE between us and the sun,
					//perform a 2nd check to see if there's anything translucent between us and the sun.
					if (texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
						//surface has translucent object between it and the sun. modify its color.
						//if the block light is high, modify the color less.
						 shadowLightColor = texture2D(shadowcolor0, shadowPos.xy);
						//make colors more intense when the shadow light color is more opaque.
						shadowLightColor.rgb = mix(vec3(1.0), shadowLightColor.rgb, shadowLightColor.a);
						//also make colors less intense when the block light level is high.
					//	shadowLightColor.rgb = mix(shadowLightColor.rgb, vec3(1.0), lm.x);
						
						
						
					}
				#endif
			}
		}

		#include "/pbr2.glsl"
		
		color.rgb = 	
		#if PBR >= 2
			sun_shine+sky_shine+
		#endif 
		color.rgb* max(vec3(MINIMUM_LIGHT_LEVEL),
		#if PBR > 0
			sun_lighting*
		#else
			 shadowPos.w* //sunlighting wo normals
		#endif
		sun_color*
		 shadowLightColor.rgb*main_lighting
		*sky_color
		+
		#if PBR > 0
			ao_allowed_light*
			(lm.y*sky_color*sky_lighting 
			+lm.x*torch_color)
		#else
			lm.y*sky_color
			+lm.x*torch_color
		#endif
		
		)
		;
	#else
		#include "/pbr2.glsl"
		color.rgb = 	
		#if PBR >= 2
			sun_shine+sky_shine+
		#endif 
		color.rgb* max(vec3(MINIMUM_LIGHT_LEVEL),

		
		
		#if DIRECTIONAL_LIGHTING == 1
			#if PBR > 0
				sun_lighting*
			#else
				 shadowPos.w*  //sunlighting wo normals
			#endif
			
			main_lighting
		#else 
			
			main_lighting
		#endif
		*sun_color
		+
	
			lm.y*sky_color
			+lm.x*torch_color

		
		);
		;
	
	#endif
	
	#if SHOW_MOB_DAMAGE == 1 && IS_AN_ENTITY == 1
		color.rgb= mix(color.rgb,entityColor4.rgb,entityColor4.a);
	#endif
	
	#if FOG == 1
		float water_fog = isEyeInWater == 1? 30. : isEyeInWater == 2? 20. : isEyeInWater == 1? 10. : 0.;
		water_fog = (water_fog > 1. ) ? clamp((dist)/water_fog,0.,1.) : 0.;
		//fogColor FOG_START FOG_END
			float border_fog_amount = clamp((dist-(BORDER_FOG_START*far))/(((1.-BORDER_FOG_START)*far)),0.,1.);
			float fog_amount = 
				clamp(
				max(
				water_fog,
				max( 
					clamp((dist-FOG_START)/(FOG_END-FOG_START),0.,FOG_MAX),
					border_fog_amount)
					*(1.+rainStrength)
					)
				,0.,1.)
				;
			color.rgb = mix(color.rgb,fogColor.rgb,fog_amount);
	#endif
	
	#if DONT_BLOW_OUT_WHITES == 1
		color.r = color.r <.95?color.r : .95+(color.r-.95)*.1;
		color.g = color.g <.95?color.g : .95+(color.g-.95)*.1;
		color.b = color.b <.95?color.b : .95+(color.b-.95)*.1;
	#endif

/* DRAWBUFFERS:0 */
	gl_FragData[0] = color; //gcolor
}